#define MWINCLUDECOLORS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <sys/inotify.h>
#include <nano-X.h>
#include <errno.h>

#define SSIZE 100
#define LINEHI 15
#define MAXLINECOUNT 20

int linenr, linecount;
char *lines[MAXLINECOUNT];
GR_TIMER_ID tid;
int pos;
GR_WINDOW_INFO root_wi, wi;

int remaining(int fs) {                     //remaining chars in the pipe
  int nbtoread, err;

  err = ioctl(fs, FIONREAD, &nbtoread);
  if (err==-1){
    printf("err ioctl\n");
  };
  return nbtoread;
};

void displaystring(char *s, GR_WINDOW_ID w, GR_GC_ID gc) {
  int h, l;

  l = strlen(s);
  if (s[l-1] == '\n') {
    l--;
    s[l] = 0;
  };
  h = 10 + (linenr*LINEHI); 
  GrText(w, gc, 10, h, s, l, GR_TFASCII); 
  linenr++;
};

int readfile(char *name) {
  FILE *f;
  int err;
  char * s1;

  f = fopen(name, "r");
  if (!f) {
    err = errno;
    printf("err: %d\n", err);
    return(err);
  };
  while (linecount<MAXLINECOUNT-1) {
    s1 =  fgets(lines [linecount], SSIZE, f);
    if (!s1){
      strncpy(lines[linecount++], "++++++++++++", SSIZE);
      break;
    };
    linecount++;
  };
  fclose(f);
  return 0;
};

void displaytext(GR_WINDOW_ID w) {
  int i;
  GR_GC_ID gc;

  gc = GrNewGC();
  GrSetGCForeground(gc, WHITE);
  GrSetGCUseBackground(gc, GR_FALSE);
  GrFillRect(w, gc, 0, 0, 300, MAXLINECOUNT*LINEHI);
  GrSetGCForeground(gc, BLACK);
  linenr = 0;
  for (i=0; i<linecount; i++) {
    displaystring(lines[i], w, gc);
  };
  GrDestroyGC(gc);
};

void dofdinput(GR_WINDOW_ID w, int fdread, char *f1, char *f2) {
  char ldummy, *dummy;
  int count;
  
  ldummy = 0;
  for(;;) {                        // read all the content of the pipe
    count = remaining(fdread);     // an inotify pipe can't be read byte by byte
                                   // but only in chunks of an appropriate size 
                                   // so wr just read all bytes
    if (!count) break;
    dummy = malloc(count);       
    read(fdread, dummy, count);
    ldummy = 1;
  };
  if (ldummy) {
    linecount = 0;
    readfile(f1);
    if (f2) readfile (f2);
  };
  displaytext(w);
};

void dotimer(GR_WINDOW_ID w) { //see nxeyes for comments on timer use
  GR_WINDOW_ID wd;
  int px, py;

  GrGetWindowInfo(w, &wi);
  wd = wi.parent;
  if (tid) GrDestroyTimer(tid);
  tid = GrCreateTimer(w, 10000);
  if (wd != GR_ROOT_WINDOW_ID) { // nanowm reparents our window with the window 
                                 // showing the decoration. So we need to move 
                                 // the decoration window. But nanowm  does this 
                                 // asynchronously and thus we can't detect the 
                                 // parent window ID right after the program start
                                 // but need to detect it it had happened later
    switch (pos++) {
     case 0: 
      px = 0;
      py = 0;
      break; 
     case 1: 
      px = root_wi.width-wi.width;
      py = 0;
      break; 
     case 2: 
      px = root_wi.width-wi.width;
      py = root_wi.height-wi.height;
      break; 
     default:
      pos = 0;
      px = 0;
      py = root_wi.height-wi.height;
      break; 
    };
    GrMoveWindow(wd, px, py);
  };
};

void settitle(GR_WINDOW_ID w) {
  const int titmaxlen = 256;
  char tit[titmaxlen], ver[titmaxlen];
  GR_WM_PROPERTIES w_properties;
  FILE *f;
  char *ci;
  int i, j, k;

  f = fopen("/proc/version", "r");
  if (f) {
    fgets(ver, titmaxlen, f);
    ci = strchr(ver, '(');
    if (ci) {
      i = ci-ver-1;
      strncpy(tit, ver, i);
      tit[i] = ',';
      tit[i+1] = 0;
      k = strlen(ver)-1;
      for (j=5; j>0; j--) {
        while (k>i) {
          if (ver[k--] == ' ') break; 
        };
      };
      strcat(tit, ver+k+1);
     } else {
      strcpy(tit, "no ( in /proc/version");
    };
   } else {
    strcpy(tit, "/proc/version error");
  };
  GrGetWMProperties(w, &w_properties);
  w_properties.title = tit;
  GrSetWMProperties(w, &w_properties);
//  tit[0] = 'X'; tit[1] = 0;
};

int main(int ac, char **av) {
  GR_WINDOW_ID w;
  GR_EVENT event;
  char *f1, *f2;
  int  fdread;
  int err, i, done;

  av++;
  f1 = *av++;
  if (!f1) return -1;
  f2 = *av;

  fdread = inotify_init();
  if (fdread==-1){
    printf("inotify_init err: %s\n", strerror(errno));
    return -1;
  };

  err = inotify_add_watch(fdread, f1, IN_MODIFY);
  if (err ==-1){
    printf("inotify_add_watch err: %s\n", strerror(errno));
    return -1;
  };

  if (f2) {
    err = inotify_add_watch(fdread, f2, IN_MODIFY);
    if (err ==-1){
      printf("inotify_add_watch err: %s\n", strerror(errno));
      return -1;
    };
  };

  for (i=0; i<MAXLINECOUNT; i++) {
    lines[i] = malloc(SSIZE);
  };
  linecount = 0;
  readfile(f1);
  if (f2) readfile (f2);

  if (GrOpen() < 0) {
    printf("Can't open graphics\n");
    exit(1);
  };
  w = GrNewWindow(GR_ROOT_WINDOW_ID, 20, 20, 300, MAXLINECOUNT*LINEHI, 
      4, WHITE, BLUE);
  GrGetWindowInfo(GR_ROOT_WINDOW_ID, &root_wi);
  GrSelectEvents(w, GR_EVENT_MASK_EXPOSURE|GR_EVENT_MASK_TIMER|GR_EVENT_MASK_CLOSE_REQ);
  settitle(w);

  GrMapWindow(w);                               // make the Window visible

  GrRegisterInput(fdread);                      // Event Pipe not empty: 
                                                // GR_EVENT_MASK_FDINPUT needs not to be included with GrSelectEvents
  dotimer(w);                                   // start timer
  done = 0;
  while (!done) {                               //nano-X main-loop
    GrGetNextEvent(&event);
    switch (event.type) {
     case GR_EVENT_TYPE_EXPOSURE:
      displaytext(w);
      break;
     case GR_EVENT_TYPE_FDINPUT:
      dofdinput(w, fdread, f1, f2);
      break;
     case GR_EVENT_TYPE_TIMER:
      dotimer(w);
      break;
     case GR_EVENT_TYPE_CLOSE_REQ:
      done = !0;
      break;
    };
  };
  GrClose();
  return 0;
};

